local _, L = ...;

function MiraMod_OnLoad()

	-- Register events
	MiraModEventTable				= {};	
	MiraModEventTable["COMBAT_LOG_EVENT_UNFILTERED"]	= MiraMod_CombatLog_Event;
	MiraModEventTable["VARIABLES_LOADED"]				= MiraMod_OnVariablesLoaded;
	MiraModEventTable["PLAYER_ENTERING_WORLD"]			= MiraMod_ZoneCheck;
	MiraModEventTable["ENCOUNTER_START"]				= MiraMod_Encounter_Start;
	MiraModEventTable["ENCOUNTER_END"]					= MiraMod_Encounter_End;
	MiraModEventTable["RESURRECT_REQUEST"]				= MiraMod_Resurrect_Request;
	MiraModEventTable["READY_CHECK"]					= MiraMod_ReadyCheck;
	MiraModEventTable["CHAT_MSG_MONSTER_YELL"]			= MiraMod_Monster_Yell;
	MiraModEventTable["RAID_TARGET_UPDATE"]				= MiraMod_RaidTargetUpdate;
	
	for key,value in pairs(MiraModEventTable) do	
		MiraMod:RegisterEvent(key);	
	end
		
	SlashCmdList["MiraMod"] = MiraMod_OnCommand;
	SLASH_MiraMod1 = "/Miramod";
	SLASH_MiraMod2 = "/miramod";
	
	-- Init global variables
	UpdateTimeInterval 				= 0.5;
	TimeSinceLastUpdate				= 0;
	MiraModTextDisplay 				= nil;
	SoundDir 						= nil;
	nextraid_symbol 				= 8;
	
	MiraModInstances				= {};	-- List of instances
											-- key: 	 instance name 
											-- elements: active			Boolean that contains the info if the init function has been called yet	
											--			 initFunc		Function that would add the debuffs/combat entries to the scanner
											--							When the instance has been entered
											
	MiraModTimers					= {};	-- List of active timers
											-- elements: endTime		Contains the timestamp when the timer rans out
											-- 			 msg			Message that is displayed
											--			 snd			Sound that is played
											--			 func			Function that is called
		
	MiraModDebuffs					= {};	-- List of debuffs
											-- elements: name			Name of the debuff
											--			 state			True = active, false = not active
											--			 addSound		Played soundfile when debuff is detected
											--			 addMsg			Displayed text message
											--			 removeSound	Played soundfile when debuff is removed
											--			 removeMsg		Displayed text message
											--			 scanInGroup	True if this debuff has to be searched in the group, to find special debuffs like blackout
											--			 addYell		Text which is yelled when debuff is detected
											--			 removeYell		Text which is yelled when debuff is removed
											--			 stackCount		Warning will be only fired when the debuff reaches this or more stack count
											--			 triggersCount	True, if this debuffs causes an countdown
											--			 countdown		Used for countdown
											--			 countdownPos	Position for countdown
		
	MiraModCombat					= {};	-- List of combat events
											-- elements: source			caster
											-- 			 actionType		can be SPELL_CAST_SUCCESS, SPELL_AURA_APPLIED, SPELL_AURA_REMOVED
											--			 spellId		Id of the spell, if known
											--			 spellName		Name of the spell
											--			 textMsg		Text message which is displayed
											--			 func			Function that is called
											--			 stackCount		Stackcount
											--			 target			Target of the spell
		
	-- List of functions that will be called ...
	MiraModYellCheckFunctions		= {};	-- ... when a NPC is yelling
	MiraModRaidTargetFunctions		= {};	-- ... when raid targets are updated
	MiraModEncounterStartFunctions	= {};	-- ... when encounter starts
			
end

function MiraMod_PlaySound(file)

	-- MiraMod_Write("PlaySound: " .. SoundDir .. file);

	PlaySoundFile(SoundDir .. file .. ".ogg", "Master");

end

function MiraMod_PlayCommonSound(file)

	local fullname = "Interface\\AddOns\\MiraMod\\Sounds\\common\\" .. file;
	
	PlaySoundFile(fullname, "Master");

end

function MiraMod_GetTranslation(id)

	return L[id];

end

function MiraMod_OnCommand(arg)

	local str = L[arg];
	MiraMod_Write("L["..arg.."] = "..str);
	
	if(arg == "combat") then
	
		local CombatCount = table.getn(MiraModCombat);
		
		for za=1,CombatCount do
		
			local entry = MiraModCombat[za];
		
			if(entry["source"] ~= nil and entry["spellName"] ~= nil) then
			
				src 	= MiraMod_CheckParam(entry["source"]);
				spell 	= MiraMod_CheckParam(entry["spellName"]);
				act 	= MiraMod_CheckParam(entry["actionType"]);
				dest 	= MiraMod_CheckParam(entry["target"]);
				
				local msg = "Source["..za.."] = "..src..", Spell["..za.."] = "..spell..", Action = "..act..", Target: "..dest;
				MiraMod_Write(msg);

			end
		
		end
	end
	
	if(arg == "debuff") then
	
		local DebuffCount = table.getn(MiraModDebuffs);
		
		for za=1,DebuffCount do
		
			if(MiraModDebuffs[za] ~= nil) then
			
				local entry = MiraModDebuffs[za];
				local msg = "Name[" .. za .. "] = " .. entry["name"];
				MiraMod_Write(msg);
						
			end
		
		end	
	
	end
	
	if(arg == "ini") then
	
		for key,value in pairs(MiraModInstances) do	
			MiraMod_Write("Ini: " .. key .. ", active = " .. MiraMod_BoolToString(MiraModInstances[key]["active"]));
		end
	
	end

end

function MiraMod_TestTimer()

	MiraMod_AddSpecialTimer(time() + 5, "Test Timer ran out", "gouge", MiraMod_SetRaidSymbol);

end

function MiraMod_OnVariablesLoaded()

	MiraModTextDisplay = _G["MiraModTextDisplay"];

	MiraModTextDisplay:Show();
	MiraModTextDisplay:SetFadeDuration(1);
	MiraModTextDisplay:SetTimeVisible(2);
	
	if(SoundDir == nil) then
		SoundDir = "Interface\\AddOns\\MiraMod\\Sounds\\en";
	end
		
	SoundDir = SoundDir .. "\\";
	
	MiraMod_AddInstance("BlackwingDescent", MiraMod_BlackwingDescent_Init);
	MiraMod_AddInstance("TheBastionofTwilight", MiraMod_TheBastionofTwilight_Init);
	MiraMod_AddInstance("IcecrownCitadel", MiraMod_IcecrownCitadel_Init);
	MiraMod_AddInstance("Firelands", MiraMod_Firelands_Init);
	MiraMod_AddInstance("MogushanVaults", MiraMod_MogushanVaults_Init);
	MiraMod_AddInstance("ThroneoftheFourWinds", MiraMod_FourWinds_Init);
	MiraMod_AddInstance("HeartofFear", MiraMod_HeartofFear_Init);
	MiraMod_AddInstance("DragonSoul", MiraMod_DragonSoul_Init);
	MiraMod_AddInstance("TheRubySanctum", MiraMod_RubySanctum_Init);
	MiraMod_AddInstance("SiegeofOrgrimmar", MiraMod_SiegeofOrgrimmar_Init);
		
	-- Test
	-- function MiraMod_AddDebuff(name, addSound, addTextMsg, removeSound, removeMsg, scanInGroup, addYell, removeYell, stackCount, tiggerCountdown, countDown)
	-- MiraMod_AddDebuff("Weakened Soul", "seuche", "Weakened Soul Get", "verbindungweg", "Weakened Soul Rem", false, "Chogall_Corruption_Sickness_Yell_Add", nil, 0, true, 13);			
	-- MiraMod_AddCombat("Dorijana", "SPELL_AURA_APPLIED", 0, "Power Word Fortitude", "lauf", "Power Word Fortitude", MiraMod_TestTimer);
	
	-- Minor instances
	
	-- Evolution (Corla, Botin des Zwielichts, Schwarzfelshhlen)
	MiraMod_AddDebuff("Corla_Evolution", nil, nil, "verbindungweg", "Corla_Evolution_Rem");
							
	-- Kugelgefngnis (Hallen des Urspungs Trash)
	MiraMod_AddDebuff("BubbleBound", "seuche", "BubbleBound_Add", "verbindungweg", "BubbleBound_Rem", false, "BubbleBound_Yell_Add", "BubbleBound_Yell_Rem");
		
	-- Schattenvortex (Burg Schattenfang Trash)
	MiraMod_AddDebuff("Shadowvortex", "lauf", "Shadowvortex_Add", "verbindungweg", "Shadowvortex_Rem");	
	
	-- Statische Aufladung (Asaad, Vortex Gipfel)
	MiraMod_AddCombat("Asaad", "SPELL_CAST_START", 0, "Asaad_Static_Cling", "seuche", "Asaad_Static_Cling_Msg");
	
	MiraMod_AddYellCheckFunction(MiraMod_MinorYell);
		
	-- Say hello		
	if(DEFAULT_CHAT_FRAME ~= nil) then
		MiraMod_Write(L["StartUpMsg"], 0.5, 0.5, 1.0);
	else
		MiraModTextDisplay:AddMessage(L["StartUpMsg"], 0.5, 0.5, 1.0);	
	end
	
end

function MiraMod_MinorYell(arg1, arg2)

	-- Cata
	if(arg2 == L["Ozruk"]) then
	
		if(arg1 == L["Ozruk_Reflect"]) then
			MiraMod_PlaySound("seuche");
			MiraModTextDisplay:AddMessage(L["Ozruk_Reflect_Msg"], 1.0, 0.32, 0.66);
		end
		
	end

end

function MiraMod_SetRaidSymbol(Target)

	if(Target ~= nil) then
		
		local icon = GetRaidTargetIndex(Target);
		
		if(icon ~= nextraid_symbol) then 
			SetRaidTargetIcon(Target, nextraid_symbol);
		end
		
		if(nextraid_symbol == 8) then
		
			MiraMod_AddSpecialTimer(time() + 1, nil, nil, MiraMod_OnResetRaidSymbol);
				
		end
	end 
	
	nextraid_symbol = nextraid_symbol - 1;

	if(nextraid_symbol<0) then
		nextraid_symbol = 8;
	end

end

function MiraMod_RemoveRaidSymbol(Target)

	if(Target ~= nil) then
	
		local icon = GetRaidTargetIndex(Target);
		if(icon ~= nil) then
		
			SetRaidTargetIcon(Target, icon);
		
		end
	
	end

end

function MiraMod_OnResetRaidSymbol()

	nextraid_symbol = 8;

end

function MiraMod_OnUpdate(self, elapsed)

	if(MiraModTextDisplay) then

		TimeSinceLastUpdate = TimeSinceLastUpdate + elapsed;

		if (TimeSinceLastUpdate > UpdateTimeInterval) then
		
			local numplayers = GetNumGroupMembers();
	
			local za = 0;
			local debuffCount = table.getn(MiraModDebuffs);
				
			for za=1, debuffCount do
													
				local Debuff, Rank, Icon, Count, DebuffType, Duration, expirationTime;
				
				if(MiraModDebuffs[za]["scanInGroup"] == false) then

					if(MiraModDebuffs[za]["name"] ~= nil) then
					
						Debuff, Rank, Icon, Count, DebuffType, Duration, expirationTime = UnitDebuff("player", MiraModDebuffs[za]["name"]);
					
						if(Debuff == nil) then
							Debuff, Rank, Icon, Count, DebuffType, Duration, expirationTime = UnitBuff("player", MiraModDebuffs[za]["name"]);
						end
							
					end	
				
				else
					
					for zb=1,numplayers do
						
						local name = GetRaidRosterInfo(zb);
						
						if(name ~= nil) then
							Debuff, Rank, Icon, Count = UnitDebuff(name, MiraModDebuffs[za]["name"]);
							if(Debuff ~= nil) then
								break
							end
						end
							
					end
					
				end
				
				if(Debuff) then
								
					if(not MiraModDebuffs[za]["state"] and (MiraModDebuffs[za]["stackCount"]==0 or Count >= MiraModDebuffs[za]["stackCount"])) then

						if(MiraModDebuffs[za]["addSound"] ~= nil) then 
							MiraMod_PlaySound(MiraModDebuffs[za]["addSound"]);
						end
						
						if(MiraModDebuffs[za]["addMsg"] ~= nil) then
							MiraModTextDisplay:AddMessage(MiraModDebuffs[za]["addMsg"], 1.0, 0.32, 0.66);
						end
						
						if(MiraModDebuffs[za]["addYell"] ~= nil) then
							SendChatMessage(MiraModDebuffs[za]["addYell"], "YELL", nil, 0);
						end
					
						MiraModDebuffs[za]["state"] = true;
						
						if(MiraModDebuffs[za]["triggersCount"] == true) then
							MiraModDebuffs[za]["countdown"] = math.floor(expirationTime - GetTime()) - 1;
						end						

						if(MiraModDebuffs[za]["countdown"] > 0) then
							MiraModDebuffs[za]["countdownPos"] = MiraModDebuffs[za]["countdown"];
						end
						
					end
				end
								
				if(MiraModDebuffs[za]["countdownPos"] ~= nil) then
							
					if(MiraModDebuffs[za]["countdownPos"]>UpdateTimeInterval) then
					
						local countPos, fraction = math.modf(MiraModDebuffs[za]["countdownPos"]);
										
						local countPos = math.floor(MiraModDebuffs[za]["countdownPos"]);
						if(countPos<10 and countPos > 0 and fraction == 0) then
							MiraMod_PlaySound(countPos);
						end						
												
						MiraModDebuffs[za]["countdownPos"] = MiraModDebuffs[za]["countdownPos"] - UpdateTimeInterval;
						
					end
				
				end
				
				if(not Debuff and MiraModDebuffs[za]["state"]==true) then
				
					MiraModDebuffs[za]["state"] = false;
				
					if(MiraModDebuffs[za]["removeMsg"] ~= nil) then					
						MiraModTextDisplay:AddMessage(MiraModDebuffs[za]["removeMsg"], 0.32, 1.00, 0.32);					
					end
					
					if(MiraModDebuffs[za]["removeSound"] ~= nil) then
						MiraMod_PlaySound(MiraModDebuffs[za]["removeSound"]);
					end
					
					if(MiraModDebuffs[za]["removeYell"] ~= nil) then
						SendChatMessage(MiraModDebuffs[za]["removeYell"], "YELL", nil, 0);
					end						
					
					if(MiraModDebuffs[za]["countdownPos"]>0) then
						MiraModDebuffs[za]["countdownPos"] = 0;
					end
					
				end
				
			end
				
			local TimerSize 	= #MiraModTimers;
			local CurrentTime 	= time();
								
			for za=TimerSize,1,-1 do
			
				local entry   = MiraModTimers[za];
				local endTime = entry["endTime"];
											
				if(endTime <= CurrentTime) then
				
					-- Timer ran out
					local Msg = entry["msg"];
					local Snd = entry["snd"];
					local Func= entry["func"];
					
					if(Msg ~= nil) then
						MiraModTextDisplay:AddMessage(Msg, 1.0, 1.0, 0.55);
					end
																	
					if(Snd ~= nil) then					
						MiraMod_PlaySound(Snd);
					end
					
					if(Func ~= nil) then				
						Func();					
					end
					
					table.remove(MiraModTimers, za);
				
				end
			
			end
			
			TimeSinceLastUpdate = 0;
		
		end

	end

end

function MiraMod_ZoneCheck()

	local inInstance = IsInInstance();
			
	if(inInstance) then
	
		local name = GetInstanceInfo();
		
		MiraMod_Write("InstanceName: " .. name);
		
		local entry = MiraModInstances[name];		
		if(entry ~= nil) then
			if(entry["active"] == false) then
				entry["active"] = true;
				entry["initFunc"]();
			end
		end
	end
end

function MiraMod_CombatLog_Event(arg1, action, arg3, arg4, src, arg6, arg7, arg8, dest, arg10, arg11, spellId, spellName, arg14, arg15, stackCount, arg17, arg18)
	
	local za = 0;
	local found = false;
	local CombatCount = table.getn(MiraModCombat);
						
	for za=1, CombatCount do
		
		if(action == MiraModCombat[za]["actionType"]) then
					
			local spellCheck 	= (MiraModCombat[za]["spellId"] 	== spellId 		or  MiraModCombat[za]["spellName"] 	== spellName);			
			local sourceCheck 	= MiraModCombat[za]["source"] 		== src 			or  MiraModCombat[za]["source"] 	== nil;
			local diedCheck		= MiraModCombat[za]["actionType"] 	== "UNIT_DIED" 	and MiraModCombat[za]["source"] 	== dest;
												
			if (spellCheck and sourceCheck) or (diedCheck) then
										
				if(MiraModCombat[za]["stackCount"] == 0 or MiraModCombat[za]["stackCount"] == stackCount) then
																	
					if(MiraModCombat[za]["target"] == nil or MiraModCombat[za]["target"] == dest) then
																
						found = true;
							
						if(MiraModCombat[za]["sound"] ~= nil) then 
							MiraMod_PlaySound(MiraModCombat[za]["sound"]);
						end
						
						if(MiraModCombat[za]["textMsg"] ~= nil) then
							MiraModTextDisplay:AddMessage(MiraModCombat[za]["textMsg"], 1.0, 0.32, 0.66);
						end
						
						if(MiraModCombat[za]["func"] ~= nil) then									
							MiraModCombat[za]["func"](dest);
						end
						
						break;
					
					end
				
				end
			
			end
	
		end
	end
end

function MiraMod_Encounter_Start(arg1, arg2, arg3, arg4)

	MiraMod_Write("Start encounter: " .. arg2);

	local EncounterStartCount = table.getn(MiraModEncounterStartFunctions);

	for za=1, EncounterStartCount do
			
		MiraModEncounterStartFunctions[za](arg1, arg2, arg3, arg4);
					
	end

end

function MiraMod_Encounter_End(arg1, arg2, arg3, arg4, arg5)

	local result = "wipe";

	if arg5 == 1 then
	
		result = "success";
		
		--MiraMod_PlayCommonSound("FF1");
	
	end

	MiraMod_Write("End encounter: " .. arg2.. ", result: " .. result);
	
	MiraMod_ClearTable(MiraModTimers);	
end

function MiraMod_Monster_Yell(arg1, arg2)

	local yellFunctionsCount = table.getn(MiraModYellCheckFunctions);

	for za=1, yellFunctionsCount do
	
		MiraModYellCheckFunctions[za](arg1, arg2);
	
	end

end

function MiraMod_RaidTargetUpdate()

	local raidTargetFunctionsCount = table.getn(MiraModRaidTargetFunctions);
	
	for za=1, raidTargetFunctionsCount do
	
		MiraModRaidTargetFunctions[za]();
	
	end

end

function MiraMod_Resurrect_Request()

	MiraMod_PlayCommonSound("extralife");

end

function MiraMod_ReadyCheck()

	MiraMod_PlayCommonSound("start");

end

function MiraMod_OnEvent(self, event, ...)

	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18 = select(1, ...)

	if MiraModTextDisplay == nil then
		return;
	end
				
	func = MiraModEventTable[event];
	if(func ~= nil) then
	
		func(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13, arg14, arg15, arg16, arg17, arg18);
	
	end
	
end